@@ -27,6 +27,7 @@
 #include <string.h>
 #include <signal.h>
 #include <gdk/gdkkeysyms.h>
+#include <math.h>
 #include "procman.h"
 #include "callbacks.h"
 #include "interface.h"
@@ -117,26 +118,6 @@ static GnomeUIInfo menubar1_uiinfo[] =
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo view_optionmenu[] = 
-{
-	{
- 	  GNOME_APP_UI_ITEM, N_("All Processes"), N_("View processes being run by all users"),
-	 cb_all_process_menu_clicked, NULL, NULL, 0, 0,
-	 't', GDK_CONTROL_MASK
-	},
-	{
- 	  GNOME_APP_UI_ITEM, N_("My Processes"), N_("View processes being run by you"),
-	 cb_my_process_menu_clicked, NULL, NULL, 0, 0,
-	 'p', GDK_CONTROL_MASK
-	},
-	{
- 	  GNOME_APP_UI_ITEM, N_("Active Processes"), N_("View only active processes"),
-	 cb_running_process_menu_clicked, NULL, NULL, 0, 0,
-	 'o', GDK_CONTROL_MASK
-	},
-	GNOMEUIINFO_END
-};
-
 static GnomeUIInfo popup_menu_uiinfo[] =
 {
 	{
@@ -184,7 +165,6 @@ get_sys_pane_pos (void)
 	return GTK_PANED (sys_pane)->child1_size;
 }
 
-
 static GtkWidget *
 create_proc_view (ProcData *procdata)
 {
@@ -192,19 +172,18 @@ create_proc_view (ProcData *procdata)
 	GtkWidget *hbox1;
 	GtkWidget *search_label;
 	GtkWidget *search_entry;
-	GtkWidget *optionmenu1;
-	GtkWidget *optionmenu1_menu;
+	GtkWidget *proc_combo;
 	GtkWidget *scrolled;
 	GtkWidget *infobox;
 	GtkWidget *label;
 	GtkWidget *hbox2;
-	
+
 	vbox1 = gtk_vbox_new (FALSE, 18);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 12);
 	
 	hbox1 = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
-	
+
 	search_label = gtk_label_new_with_mnemonic (_("Sea_rch:"));
 	gtk_box_pack_start (GTK_BOX (hbox1), search_label, FALSE, FALSE, 0);
 	
@@ -213,29 +192,31 @@ create_proc_view (ProcData *procdata)
 	g_signal_connect (G_OBJECT (search_entry), "activate",
 			  G_CALLBACK (cb_search), procdata);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (search_label), search_entry);
-	
-	optionmenu1 = gtk_option_menu_new ();
-	gtk_box_pack_end (GTK_BOX (hbox1), optionmenu1, FALSE, FALSE, 0);
-  	optionmenu1_menu = gtk_menu_new ();
 
-  	gnome_app_fill_menu_with_data (GTK_MENU_SHELL (optionmenu1_menu), view_optionmenu,
-  			               NULL, TRUE, 0, procdata);
+	proc_combo = gtk_combo_box_new_text ();
+	gtk_box_pack_end (GTK_BOX (hbox1), proc_combo, FALSE, FALSE, 0);
+
+	gtk_combo_box_insert_text (GTK_COMBO_BOX (proc_combo), ALL_PROCESSES, _("All Processes"));
+	gtk_combo_box_insert_text (GTK_COMBO_BOX (proc_combo), MY_PROCESSES, _("My Processes"));
+	gtk_combo_box_insert_text (GTK_COMBO_BOX (proc_combo), ACTIVE_PROCESSES, _("Active Processes"));
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (proc_combo), procdata->config.whose_process);
+
+	g_signal_connect (G_OBJECT (proc_combo), "changed",
+			  G_CALLBACK (cb_proc_combo_changed), procdata);
 
-	gtk_menu_set_active (GTK_MENU (optionmenu1_menu), procdata->config.whose_process);
-  	gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);
-  	  	
   	label = gtk_label_new_with_mnemonic (_("Vie_w:"));
-  	gtk_label_set_mnemonic_widget (GTK_LABEL (label), optionmenu1);
+  	gtk_label_set_mnemonic_widget (GTK_LABEL (label), proc_combo);
 	gtk_box_pack_end (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	
+
 	gtk_widget_show_all (hbox1);
-	
+
 	scrolled = proctable_new (procdata);
 	if (!scrolled)
 		return NULL;
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
                                              GTK_SHADOW_IN);
-	
+
 	gtk_box_pack_start (GTK_BOX (vbox1), scrolled, TRUE, TRUE, 0);
 	
 	gtk_widget_show_all (scrolled);
@@ -265,7 +246,7 @@ create_proc_view (ProcData *procdata)
  	gnome_app_fill_menu_with_data (GTK_MENU_SHELL (popup_menu), popup_menu_uiinfo,
   			               NULL, TRUE, 0, procdata);
 	gtk_widget_show (popup_menu);
-      
+
         return vbox1;
 }
 
@@ -322,6 +303,17 @@ make_title_label (const char *text)
   return label;
 }
 
+/* Make sure the cpu labels don't jump around. From the clock applet */
+static void
+cpu_size_request (GtkWidget *box, GtkRequisition *req, ProcData *procdata)
+{
+	if (req->width > procdata->cpu_label_fixed_width)
+		procdata->cpu_label_fixed_width = req->width;
+		
+	req->width = procdata->cpu_label_fixed_width;
+	
+}
+
 static GtkWidget *
 create_sys_view (ProcData *procdata)
 {
@@ -338,6 +330,7 @@ create_sys_view (ProcData *procdata)
 	GtkTreeStore *model;
 	GtkTreeViewColumn *col;
 	GtkCellRenderer *cell;
+	GtkSizeGroup *sizegroup;
 	gchar *titles[5] = {_("Name"),
 			    _("Directory"),
 				_("Type"),
@@ -379,30 +372,47 @@ create_sys_view (ProcData *procdata)
 	cpu_graph = load_graph_new (CPU_GRAPH, procdata);
 	gtk_box_pack_start (GTK_BOX (cpu_graph_box), cpu_graph->main_widget, TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 12);
-	gtk_box_pack_start (GTK_BOX (cpu_graph_box), hbox, FALSE, FALSE, 0);
-
-	color_picker = gnome_color_picker_new ();
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (color_picker), 
-				    cpu_graph->colors[2].red,
-				    cpu_graph->colors[2].green,
-				    cpu_graph->colors[2].blue, 0);
-	g_signal_connect (G_OBJECT (color_picker), "color_set",
-			    G_CALLBACK (cb_cpu_color_changed), procdata);
-	gtk_box_pack_start (GTK_BOX (hbox), color_picker, FALSE, FALSE, 0);
+	sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	for (i=0;i<procdata->config.num_cpus; i++) {
+		GtkWidget *temp_hbox;
+		gchar *text;
+		/* Two per row */
+		if (fabs(fmod(i,2) - 0) < .01) {
+			hbox = gtk_hbox_new (FALSE, 12);
+			gtk_box_pack_start (GTK_BOX (cpu_graph_box), hbox, FALSE, FALSE, 0);
+		}
+		
+		temp_hbox = gtk_hbox_new (FALSE, 6);
+		gtk_box_pack_start (GTK_BOX (hbox), temp_hbox, FALSE, FALSE, 0);
+		gtk_size_group_add_widget (sizegroup, temp_hbox);
+		g_signal_connect (G_OBJECT (temp_hbox), "size_request",
+					 G_CALLBACK (cpu_size_request), procdata);
+		
+		color_picker = gnome_color_picker_new ();
+		gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (color_picker), 
+				    cpu_graph->colors[2+i].red,
+				    cpu_graph->colors[2+i].green,
+				    cpu_graph->colors[2+i].blue, 0);
+		g_signal_connect (G_OBJECT (color_picker), "color_set",
+			    G_CALLBACK (cb_cpu_color_changed), GINT_TO_POINTER (i));
+		gtk_box_pack_start (GTK_BOX (temp_hbox), color_picker, FALSE, FALSE, 0);
+		
+		text = g_strdup_printf (_("CPU%d:"), i+1);
+		label = gtk_label_new (text);
+		gtk_box_pack_start (GTK_BOX (temp_hbox), label, FALSE, FALSE, 0);
+		g_free (text);
+		
+		cpu_label = gtk_label_new (NULL);
+		gtk_box_pack_start (GTK_BOX (temp_hbox), cpu_label, FALSE, FALSE, 0);
+		cpu_graph->cpu_labels[i] = cpu_label;
+		
+	}
 	
-	label = gtk_label_new (_("Used CPU:"));
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	
-	cpu_label = gtk_label_new ("");
-	gtk_box_pack_start (GTK_BOX (hbox), cpu_label, FALSE, FALSE, 0);
-	cpu_graph->label = cpu_label;
 	
 	procdata->cpu_graph = cpu_graph;
 
 
-	/******************************************/
-
 	mem_box = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), mem_box, TRUE, TRUE, 0);
 
@@ -619,7 +629,7 @@ create_main_window (ProcData *procdata)
 	
 	sys_box = create_sys_view (procdata);
 	gtk_widget_show (sys_box);
-	tab_label2 = gtk_label_new_with_mnemonic (_("Resource Monitor"));
+	tab_label2 = gtk_label_new (_("Resource Monitor"));
 	gtk_widget_show (tab_label2);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), sys_box, tab_label2);
 	
@@ -762,18 +772,23 @@ toggle_infoview (ProcData *data)
 	}
 }
 
-void do_popup_menu (ProcData *data, GdkEventButton *event)
+void
+do_popup_menu (ProcData *data, GdkEventButton *event)
 {
+	gint button;
+	gint event_time;
 
-	if (event->type == GDK_BUTTON_PRESS)
-        {
-                if (event->button == 3)
-                {
-                	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
-                                        NULL, NULL, event->button,
-                                        event->time);
-                }
-        }
+	if (event) {
+		button = event->button;
+		event_time = event->time;
+	}
+	else {
+		button = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+
+	gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL,
+			NULL, NULL, button, event_time);
 }
 
 void
